#!/usr/bin/env python
from syslog import syslog, openlog, closelog, LOG_ERR, LOG_INFO
import sys

version = '.'.join([str(x) for x in sys.version_info[:2]])
sys.path.append('/usr/local/lib/python%s/site-packages' % version)
sys.path.append('/usr/local/lib64/python%s/site-packages' % version)

import os
import subprocess
import tempfile
import config
import rrdtool
import re
import initialize_source
import traceback
import netflow_checks
import logging
import logging.handlers
import datetime

logger = logging.getLogger('backend')
hdlr = logging.handlers.RotatingFileHandler('/usr/local/nagiosna/var/backend.log', 'a', 1048576, 10)

fmt = logging.Formatter('%(asctime)s %(levelname)s : %(message)s', '%Y-%m-%d %H:%M:%S')
hdlr.setFormatter(fmt)

logger.addHandler(hdlr)
logger.setLevel(logging.DEBUG)

class Dummy(object): 
    pass

nfdump_cmd = ['/usr/local/bin/nfdump']
nfexpire_cmd = ['/usr/local/bin/nfexpire']
nfreplay_cmd = ['/usr/local/bin/nfreplay']

def update_bandwidth(directory, ABS_FLOWFILE, path):
    global nfdump_cmd
    global logger
    bandwidth_rrd = os.path.abspath(os.path.join(directory, *path))
    cmd = nfdump_cmd + ['-r', ABS_FLOWFILE, '-s', 'dstport', '-n', '1', '-N', '-o', 'csv']
    update = subprocess.Popen(cmd, stdout=subprocess.PIPE)
    update.wait()
    lines = update.stdout.readlines()
    summary = ['N'] + lines[-1].split(',')
    rrdstamp = ':'.join(summary)[:-1]
    try:
        rrdtool.update(bandwidth_rrd, rrdstamp)
    except Exception, e:
        logging.warning('Error updating the RRD: bandwidth rrd was not updated: ' + str(e))
        

def run_checks(ABS_FLOWFILE, sid):
    global logger
    logger.debug('Running checks...')
    options = Dummy()
    options.filename = ABS_FLOWFILE
    options.sourceid = sid
    try:
        netflow_checks.main(options)
        logger.info('Ran checks successfully')
    except Exception, e:
        logger.exception(e)
    
    
def test_sanity(ABS_FLOWFILE):
    assert os.path.isfile(ABS_FLOWFILE)

def update_views(directory, flowfile, sid, ABS_FLOWFILE):
    global logger
    global nfdump_cmd
    global nfexpire_cmd
    ABS_VIEWPATH = os.path.join(directory, '..', 'views')
    QUERY_SQL = '''
SELECT 
    nagiosna_Views.name, 
    nagiosna_Views.limiter,
    nagiosna_Views.lifetime
FROM 
    nagiosna_Views
JOIN 
    nagiosna_SourcesViewsLinker 
ON 
    nagiosna_SourcesViewsLinker.vid = nagiosna_Views.vid
WHERE
    nagiosna_SourcesViewsLinker.sid = %s;''' % sid 
    views = config.query_database(QUERY_SQL)
    for view in views:
        name, limiter, lifetime = view
        name = config.safe_filename(name)
        target_dir = os.path.join(ABS_VIEWPATH, name)
        if not os.path.isdir(target_dir):
            logger.warning('Target dir %s, not present. Creating it...' % target_dir)
            os.makedirs(target_dir)
        target = os.path.join(target_dir, flowfile)
        nfdc = nfdump_cmd + ['-r', ABS_FLOWFILE, '-w', target, limiter]
        nfex = nfexpire_cmd + ['-t', lifetime, '-e', os.path.join(ABS_VIEWPATH, name)]
        nfdump = subprocess.Popen(nfdc)
        nfexpire = subprocess.Popen(nfex)
        nfdump.wait()
        nfexpire.wait()
        
        if nfdump.returncode != 0:
            logger.error("nfdump command: %s, did not finish properly" % ' '.join(nfcd))
        if nfexpire.returncode != 0:
            logger.error("nfexpire command: %s, did not finish properly" % ' '.join(nfex))
        
        if not os.path.isfile(os.path.join(ABS_VIEWPATH, '%s-bandwidth.rrd' % name)):
            logger.warning('Bandwidth RRD does not seem to exist, creating it...')
            initialize_source.create_bandwidth_db(ABS_VIEWPATH, '%s-bandwidth.rrd' % name)
        path = ['..', 'views', '%s-bandwidth.rrd' % name]
        update_bandwidth(directory, target, path)

def main():
    global logger
    #~ We __must__ get the arguments passed from the nfcapd/sfcapd process
    #~ so don't even catch the exception if it gets thrown.
    directory = sys.argv[1]
    flowfile  = sys.argv[2]
    sid = sys.argv[3]
    
    logger.info('Parsing data for the source id: %s' % str(sid))
    logger.debug('Arguments: %s, %s, %s' % (directory, flowfile, int(sid)))
    
    ABS_FLOWFILE = os.path.join(directory, flowfile)
    
    test_sanity(ABS_FLOWFILE)
    mainrrd = ['..', 'bandwidth.rrd']
    try:
        update_bandwidth(directory, ABS_FLOWFILE, mainrrd)
    except Exception, e:
        logger.exception(e)
        raise e
    update_views(directory, flowfile, sid, ABS_FLOWFILE)
    run_checks(ABS_FLOWFILE, sid)

if __name__ == "__main__":
    openlog('nnabackend')
    try:
        main()
        logger.info("Successfully reaped nfcapd file.")
    except Exception, e:
        etype, evalue, tback = sys.exc_info()
        logger.error("Error reaping file: %s: %s" % (str(e), str(traceback.format_exc())))
